@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2015-2016 Canonical Ltd
+ * Copyright (C) 2015-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -94,11 +94,19 @@ var typeRegistry = map[string]*AssertionType{
 	AccountKeyRequestType.Name:    AccountKeyRequestType,
 }
 
+// Type returns the AssertionType with name or nil
+func Type(name string) *AssertionType {
+	return typeRegistry[name]
+}
+
 var maxSupportedFormat = map[string]int{}
 
 func init() {
 	// register maxSupportedFormats while breaking initialisation loop
-	maxSupportedFormat[SnapDeclarationType.Name] = 1
+
+	// 1: plugs and slots
+	// 2: support for $SLOT()/$PLUG()/$MISSING
+	maxSupportedFormat[SnapDeclarationType.Name] = 2
 }
 
 func MockMaxSupportedFormat(assertType *AssertionType, maxFormat int) (restore func()) {
@@ -109,9 +117,22 @@ func MockMaxSupportedFormat(assertType *AssertionType, maxFormat int) (restore f
 	}
 }
 
-// Type returns the AssertionType with name or nil
-func Type(name string) *AssertionType {
-	return typeRegistry[name]
+var formatAnalyzer = map[*AssertionType]func(headers map[string]interface{}, body []byte) (formatnum int, err error){
+	SnapDeclarationType: snapDeclarationFormatAnalyze,
+}
+
+// SuggestFormat returns a minimum format that supports the features that would be used by an assertion with the given components.
+func SuggestFormat(assertType *AssertionType, headers map[string]interface{}, body []byte) (formatnum int, err error) {
+	analyzer := formatAnalyzer[assertType]
+	if analyzer == nil {
+		// no analyzer, format 0 is all there is
+		return 0, nil
+	}
+	formatnum, err = analyzer(headers, body)
+	if err != nil {
+		return 0, fmt.Errorf("assertion %s: %v", assertType.Name, err)
+	}
+	return formatnum, nil
 }
 
 // Ref expresses a reference to an assertion.
@@ -310,15 +331,17 @@ var _ Assertion = (*assertionBase)(nil)
 // where:
 //
 //    HEADER is a set of header entries separated by "\n"
-//    BODY can be arbitrary,
+//    BODY can be arbitrary text,
 //    SIGNATURE is the signature
 //
+// Both BODY and HEADER must be UTF8.
+//
 // A header entry for a single line value (no '\n' in it) looks like:
 //
 //   NAME ": " SIMPLEVALUE
 //
 // The format supports multiline text values (with '\n's in them) and
-// lists possibly nested with string scalars in them.
+// lists or maps, possibly nested, with string scalars in them.
 //
 // For those a header entry looks like:
 //
@@ -327,12 +350,17 @@ var _ Assertion = (*assertionBase)(nil)
 // where MULTI can be
 //
 // * (baseindent + 4)-space indented value (multiline text)
+//
 // * entries of a list each of the form:
 //
-//   " "*baseindent "  -"  ( " " SIMPLEVALUE | "\n" MULTI )
+//     " "*baseindent "  -"  ( " " SIMPLEVALUE | "\n" MULTI )
+//
+// * entries of map each of the form:
+//
+//     " "*baseindent "  " NAME ":"  ( " " SIMPLEVALUE | "\n" MULTI )
 //
 // baseindent starts at 0 and then grows with nesting matching the
-// previous level introduction (the " "*baseindent " -" bit)
+// previous level introduction (e.g. the " "*baseindent " -" bit)
 // length minus 1.
 //
 // In general the following headers are mandatory:
@@ -348,8 +376,10 @@ var _ Assertion = (*assertionBase)(nil)
 //
 //   revision (a positive int)
 //   body-length (expected to be equal to the length of BODY)
+//   format (a positive int for the format iteration of the type used)
 //
 // Times are expected to be in the RFC3339 format: "2006-01-02T15:04:05Z07:00".
+//
 func Decode(serializedAssertion []byte) (Assertion, error) {
 	// copy to get an independent backstorage that can't be mutated later
 	assertionSnapshot := make([]byte, len(serializedAssertion))
@@ -706,6 +736,15 @@ func assembleAndSign(assertType *AssertionType, headers map[string]interface{},
 		return nil, fmt.Errorf("cannot sign %q assertion with format %d higher than max supported format %d", assertType.Name, formatnum, assertType.MaxSupportedFormat())
 	}
 
+	suggestedFormat, err := SuggestFormat(assertType, finalHeaders, finalBody)
+	if err != nil {
+		return nil, err
+	}
+
+	if suggestedFormat > formatnum {
+		return nil, fmt.Errorf("cannot sign %q assertion with format set to %d lower than min format %d covering included features", assertType.Name, formatnum, suggestedFormat)
+	}
+
 	revision, err := checkRevision(finalHeaders)
 	if err != nil {
 		return nil, err
