@@ -30,25 +30,15 @@ import (
 	"net/url"
 	"os"
 	"path"
-	"syscall"
 	"time"
 
 	"github.com/snapcore/snapd/dirs"
 )
 
-func unixDialer() func(string, string) (net.Conn, error) {
-	// We have two sockets available: the SnapdSocket (which provides
-	// administrative access), and the SnapSocket (which doesn't). Use the most
-	// powerful one available (e.g. from within snaps, SnapdSocket is hidden by
-	// apparmor unless the snap has the snapd-control interface).
-	socketPath := dirs.SnapdSocket
-	file, err := os.OpenFile(socketPath, os.O_RDWR, 0666)
-	if err == nil {
-		file.Close()
-	} else if e, ok := err.(*os.PathError); ok && (e.Err == syscall.ENOENT || e.Err == syscall.EACCES) {
-		socketPath = dirs.SnapSocket
+func unixDialer(socketPath string) func(string, string) (net.Conn, error) {
+	if socketPath == "" {
+		socketPath = dirs.SnapdSocket
 	}
-
 	return func(_, _ string) (net.Conn, error) {
 		return net.Dial("unix", socketPath)
 	}
@@ -67,6 +57,9 @@ type Config struct {
 	// DisableAuth controls whether the client should send an
 	// Authorization header from reading the auth.json data.
 	DisableAuth bool
+
+	// Socket is the path to the unix socket to use
+	Socket string
 }
 
 // A Client knows how to talk to the snappy daemon.
@@ -91,7 +84,7 @@ func New(config *Config) *Client {
 				Host:   "localhost",
 			},
 			doer: &http.Client{
-				Transport: &http.Transport{Dial: unixDialer()},
+				Transport: &http.Transport{Dial: unixDialer(config.Socket)},
 			},
 			disableAuth: config.DisableAuth,
 		}
@@ -343,6 +336,8 @@ const (
 	ErrorKindSnapAlreadyInstalled = "snap-already-installed"
 	ErrorKindSnapNotInstalled     = "snap-not-installed"
 	ErrorKindNoUpdateAvailable    = "snap-no-update-available"
+
+	ErrorKindNotSnap = "snap-not-a-snap"
 )
 
 // IsTwoFactorError returns whether the given error is due to problems
