@@ -17,28 +17,31 @@
 #include "config.h"
 #include "seccomp-support.h"
 
-#include <errno.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
-#include <search.h>
+#include <asm/ioctls.h>
 #include <ctype.h>
-#include <stdbool.h>
-#include <stdlib.h>
-#include <sys/utsname.h>
-
-// needed for search mappings
-#include <linux/can.h>
-#include <sys/prctl.h>
-#include <sys/resource.h>
+#include <errno.h>
+#include <linux/can.h>		// needed for search mappings
 #include <sched.h>
+#include <search.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/prctl.h>
+#include <sys/quota.h>
+#include <sys/resource.h>
 #include <sys/socket.h>
 #include <sys/types.h>
+#include <sys/utsname.h>
+#include <termios.h>
+#include <xfs/xqm.h>
+#include <unistd.h>
 
 #include <seccomp.h>
 
-#include "utils.h"
-#include "secure-getenv.h"
+#include "../libsnap-confine-private/secure-getenv.h"
+#include "../libsnap-confine-private/string-utils.h"
+#include "../libsnap-confine-private/utils.h"
 
 #define sc_map_add(X) sc_map_add_kvp(#X, X)
 
@@ -96,16 +99,16 @@ static struct sc_map_list sc_map_entries;
  * sc_map_search(s)	- if found, return scmp_datum_t for key, else set errno
  * sc_map_destroy()	- destroy the hash map and linked list
  */
-static scmp_datum_t sc_map_search(char *s)
+static scmp_datum_t sc_map_search(const char *s)
 {
 	ENTRY e;
 	ENTRY *ep = NULL;
 	scmp_datum_t val = 0;
 	errno = 0;
 
-	e.key = s;
+	e.key = (char *)s;
 	if (hsearch_r(e, FIND, &ep, &sc_map_htab) == 0)
-		die("hsearch_r failed");
+		die("hsearch_r failed for %s", s);
 
 	if (ep != NULL) {
 		scmp_datum_t *val_p = NULL;
@@ -165,19 +168,32 @@ static void sc_map_init()
 
 	// man 2 socket - domain
 	sc_map_add(AF_UNIX);
+	sc_map_add(PF_UNIX);
 	sc_map_add(AF_LOCAL);
+	sc_map_add(PF_LOCAL);
 	sc_map_add(AF_INET);
+	sc_map_add(PF_INET);
 	sc_map_add(AF_INET6);
+	sc_map_add(PF_INET6);
 	sc_map_add(AF_IPX);
+	sc_map_add(PF_IPX);
 	sc_map_add(AF_NETLINK);
+	sc_map_add(PF_NETLINK);
 	sc_map_add(AF_X25);
+	sc_map_add(PF_X25);
 	sc_map_add(AF_AX25);
+	sc_map_add(PF_AX25);
 	sc_map_add(AF_ATMPVC);
+	sc_map_add(PF_ATMPVC);
 	sc_map_add(AF_APPLETALK);
+	sc_map_add(PF_APPLETALK);
 	sc_map_add(AF_PACKET);
+	sc_map_add(PF_PACKET);
 	sc_map_add(AF_ALG);
+	sc_map_add(PF_ALG);
 	// linux/can.h
 	sc_map_add(AF_CAN);
+	sc_map_add(PF_CAN);
 
 	// man 2 socket - type
 	sc_map_add(SOCK_STREAM);
@@ -283,6 +299,25 @@ static void sc_map_init()
 	sc_map_add(CLONE_NEWUSER);
 	sc_map_add(CLONE_NEWUTS);
 
+	// man 4 tty_ioctl
+	sc_map_add(TIOCSTI);
+
+	// man 2 quotactl (with what Linux supports)
+	sc_map_add(Q_SYNC);
+	sc_map_add(Q_QUOTAON);
+	sc_map_add(Q_QUOTAOFF);
+	sc_map_add(Q_GETFMT);
+	sc_map_add(Q_GETINFO);
+	sc_map_add(Q_SETINFO);
+	sc_map_add(Q_GETQUOTA);
+	sc_map_add(Q_SETQUOTA);
+	sc_map_add(Q_XQUOTAON);
+	sc_map_add(Q_XQUOTAOFF);
+	sc_map_add(Q_XGETQUOTA);
+	sc_map_add(Q_XSETQLIM);
+	sc_map_add(Q_XGETQSTAT);
+	sc_map_add(Q_XQUOTARM);
+
 	// initialize the htab for our map
 	memset((void *)&sc_map_htab, 0, sizeof(sc_map_htab));
 	if (hcreate_r(sc_map_entries.count, &sc_map_htab) == 0)
@@ -320,7 +355,7 @@ static void sc_map_destroy()
 }
 
 /* Caller must check if errno != 0 */
-static scmp_datum_t read_number(char *s)
+static scmp_datum_t read_number(const char *s)
 {
 	scmp_datum_t val = 0;
 
@@ -642,8 +677,8 @@ scmp_filter_ctx sc_prepare_seccomp_context(const char *filter_profile)
 		    secure_getenv("SNAPPY_LAUNCHER_SECCOMP_PROFILE_DIR");
 
 	char profile_path[512];	// arbitrary path name limit
-	must_snprintf(profile_path, sizeof(profile_path), "%s/%s",
-		      filter_profile_dir, filter_profile);
+	sc_must_snprintf(profile_path, sizeof(profile_path), "%s/%s",
+			 filter_profile_dir, filter_profile);
 
 	f = fopen(profile_path, "r");
 	if (f == NULL) {
@@ -757,3 +792,8 @@ void sc_load_seccomp_context(scmp_filter_ctx ctx)
 			die("dropping privs after seccomp_load did not work");
 	}
 }
+
+void sc_cleanup_seccomp_release(scmp_filter_ctx * ptr)
+{
+	seccomp_release(*ptr);
+}
