@@ -35,15 +35,15 @@ import (
 	"github.com/snapcore/snapd/client"
 	"github.com/snapcore/snapd/cmd"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/httputil"
 	"github.com/snapcore/snapd/i18n"
 	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
-	"github.com/snapcore/snapd/store"
 )
 
 func init() {
 	// set User-Agent for when 'snap' talks to the store directly (snap download etc...)
-	store.SetUserAgentFromVersion(cmd.Version, "snap")
+	httputil.SetUserAgentFromVersion(cmd.Version, "snap")
 }
 
 // Standard streams, redirected for testing.
@@ -209,7 +209,10 @@ snaps on the system. Start with 'snap list' to see installed snaps.
 }
 
 // ClientConfig is the configuration of the Client used by all commands.
-var ClientConfig client.Config
+var ClientConfig = client.Config{
+	// we need the powerful snapd socket
+	Socket: dirs.SnapdSocket,
+}
 
 // Client returns a new client using ClientConfig as configuration.
 func Client() *client.Client {
