@@ -51,7 +51,7 @@ type Response interface {
 type resp struct {
 	Status int          `json:"status-code"`
 	Type   ResponseType `json:"type"`
-	Result interface{}  `json:"result"`
+	Result interface{}  `json:"result,omitempty"`
 	*Meta
 }
 
@@ -128,6 +128,11 @@ const (
 	errorKindSnapAlreadyInstalled  = errorKind("snap-already-installed")
 	errorKindSnapNotInstalled      = errorKind("snap-not-installed")
 	errorKindSnapNoUpdateAvailable = errorKind("snap-no-update-available")
+
+	errorKindNotSnap = errorKind("snap-not-a-snap")
+
+	errorKindSnapNeedsMode          = errorKind("snap-needs-mode")
+	errorKindSnapNeedsClassicSystem = errorKind("snap-needs-classic-system")
 )
 
 type errorValue interface{}
