@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -25,10 +25,9 @@ import (
 	"github.com/snapcore/snapd/interfaces"
 )
 
-var fwupdPermanentSlotAppArmor = []byte(`
-# Description: Allow operating as the fwupd service. Reserved because this
-# gives privileged access to the system.
-# Usage: reserved
+const fwupdPermanentSlotAppArmor = `
+# Description: Allow operating as the fwupd service. This gives privileged
+# access to the system.
 
   # Allow read/write access for old efivars sysfs interface
   capability sys_admin,
@@ -84,12 +83,11 @@ var fwupdPermanentSlotAppArmor = []byte(`
   dbus (bind)
       bus=system
       name="org.freedesktop.fwupd",
-`)
+`
 
-var fwupdConnectedPlugAppArmor = []byte(`
-# Description: Allow using fwupd service. Reserved because this gives
-# privileged access to the fwupd service.
-# Usage: reserved
+const fwupdConnectedPlugAppArmor = `
+# Description: Allow using fwupd service. This gives # privileged access to the
+# fwupd service.
 
   #Can access the network
   #include <abstractions/nameservice>
@@ -110,12 +108,11 @@ var fwupdConnectedPlugAppArmor = []byte(`
       path=/
       interface=org.freedesktop.DBus.Properties
       peer=(label=###SLOT_SECURITY_TAGS###),
-`)
+`
 
-var fwupdConnectedSlotAppArmor = []byte(`
-# Description: Allow firmware update using fwupd service. Reserved because this gives
-# privileged access to the fwupd service.
-# Usage: reserved
+const fwupdConnectedSlotAppArmor = `
+# Description: Allow firmware update using fwupd service. This gives privileged
+# access to the fwupd service.
 
   # Allow traffic to/from org.freedesktop.DBus for fwupd service
   dbus (receive, send)
@@ -142,9 +139,9 @@ var fwupdConnectedSlotAppArmor = []byte(`
       path=/org/freedesktop/fwupd{,/**}
       interface=org.freedesktop.fwupd
       peer=(label=###PLUG_SECURITY_TAGS###),
-`)
+`
 
-var fwupdPermanentSlotDBus = []byte(`
+const fwupdPermanentSlotDBus = `
 <policy user="root">
     <allow own="org.freedesktop.fwupd"/>
     <allow send_destination="org.freedesktop.fwupd" send_interface="org.freedesktop.fwupd"/>
@@ -156,35 +153,20 @@ var fwupdPermanentSlotDBus = []byte(`
     <deny own="org.freedesktop.fwupd"/>
     <deny send_destination="org.freedesktop.fwupd" send_interface="org.freedesktop.fwupd"/>
 </policy>
-`)
+`
 
-var fwupdPermanentSlotSecComp = []byte(`
-# Description: Allow operating as the fwupd service. Reserved because this
-# gives privileged access to the system.
-# Usage: reserved
+const fwupdPermanentSlotSecComp = `
+# Description: Allow operating as the fwupd service. This gives privileged
+# access to the system.
 # Can communicate with DBus system service
 bind
-getsockname
-recvfrom
-recvmsg
-sendmsg
-sendto
-setsockopt
-`)
+`
 
-var fwupdConnectedPlugSecComp = []byte(`
+const fwupdConnectedPlugSecComp = `
 # Description: Allow using fwupd service. Reserved because this gives
 # privileged access to the fwupd service.
-# Usage: reserved
 bind
-getsockname
-getsockopt
-recvfrom
-recvmsg
-sendmsg
-sendto
-setsockopt
-`)
+`
 
 // FwupdInterface type
 type FwupdInterface struct{}
@@ -205,10 +187,10 @@ func (iface *FwupdInterface) ConnectedPlugSnippet(plug *interfaces.Plug, slot *i
 	case interfaces.SecurityAppArmor:
 		old := []byte("###SLOT_SECURITY_TAGS###")
 		new := slotAppLabelExpr(slot)
-		snippet := bytes.Replace(fwupdConnectedPlugAppArmor, old, new, -1)
+		snippet := bytes.Replace([]byte(fwupdConnectedPlugAppArmor), old, new, -1)
 		return snippet, nil
 	case interfaces.SecuritySecComp:
-		return fwupdConnectedPlugSecComp, nil
+		return []byte(fwupdConnectedPlugSecComp), nil
 	}
 	return nil, nil
 }
@@ -217,11 +199,11 @@ func (iface *FwupdInterface) ConnectedPlugSnippet(plug *interfaces.Plug, slot *i
 func (iface *FwupdInterface) PermanentSlotSnippet(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
 	switch securitySystem {
 	case interfaces.SecurityAppArmor:
-		return fwupdPermanentSlotAppArmor, nil
+		return []byte(fwupdPermanentSlotAppArmor), nil
 	case interfaces.SecurityDBus:
-		return fwupdPermanentSlotDBus, nil
+		return []byte(fwupdPermanentSlotDBus), nil
 	case interfaces.SecuritySecComp:
-		return fwupdPermanentSlotSecComp, nil
+		return []byte(fwupdPermanentSlotSecComp), nil
 	}
 	return nil, nil
 }
@@ -232,7 +214,7 @@ func (iface *FwupdInterface) ConnectedSlotSnippet(plug *interfaces.Plug, slot *i
 	case interfaces.SecurityAppArmor:
 		old := []byte("###PLUG_SECURITY_TAGS###")
 		new := plugAppLabelExpr(plug)
-		snippet := bytes.Replace(fwupdConnectedSlotAppArmor, old, new, -1)
+		snippet := bytes.Replace([]byte(fwupdConnectedSlotAppArmor), old, new, -1)
 		return snippet, nil
 	}
 	return nil, nil
