@@ -20,16 +20,25 @@
 package builtin
 
 import (
-	"github.com/ubuntu-core/snappy/interfaces"
+	"github.com/snapcore/snapd/interfaces"
 )
 
 // http://bazaar.launchpad.net/~ubuntu-security/ubuntu-core-security/trunk/view/head:/data/apparmor/policygroups/ubuntu-core/16.04/log-observe
 const logObserveConnectedPlugAppArmor = `
-# Description: Can read system logs.
-# Usage: reserved
+# Description: Can read system logs and set kernel log rate-limiting
 
 /var/log/ r,
 /var/log/** r,
+/run/log/journal/ r,
+/run/log/journal/** r,
+/var/lib/systemd/catalog/database r,
+
+# Allow sysctl -w kernel.printk_ratelimit=#
+/{,usr/}sbin/sysctl ixr,
+@{PROC}/sys/kernel/printk_ratelimit rw,
+
+# Allow resolving kernel seccomp denials
+/usr/bin/scmp_sys_resolver ixr,
 
 # Needed since we are root and the owner/group doesn't match :\
 # So long as we have this, the cap must be reserved.
