@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -26,10 +26,9 @@ import (
 	"github.com/snapcore/snapd/release"
 )
 
-var modemManagerPermanentSlotAppArmor = []byte(`
-# Description: Allow operating as the ModemManager service. Reserved because this
-#  gives privileged access to the system.
-# Usage: reserved
+const modemManagerPermanentSlotAppArmor = `
+# Description: Allow operating as the ModemManager service. This gives
+# privileged access to the system.
 
 # To check present devices
 /run/udev/data/* r,
@@ -47,6 +46,10 @@ var modemManagerPermanentSlotAppArmor = []byte(`
 # For ioctl TIOCSSERIAL ASYNC_CLOSING_WAIT_NONE
 capability sys_admin,
 
+# For {mbim,qmi}-proxy
+unix (bind, listen) type=stream addr="@{mbim,qmi}-proxy",
+/sys/devices/**/usb**/descriptors r,
+
 include <abstractions/nameservice>
 
 # DBus accesses
@@ -76,9 +79,9 @@ dbus (receive, send)
     path=/org/freedesktop/ModemManager1{,/**}
     interface=org.freedesktop.DBus.*
     peer=(label=unconfined),
-`)
+`
 
-var modemManagerConnectedSlotAppArmor = []byte(`
+const modemManagerConnectedSlotAppArmor = `
 # Allow connected clients to interact with the service
 
 # Allow traffic to/from our path and interface with any method
@@ -94,12 +97,11 @@ dbus (receive, send)
     path=/org/freedesktop/ModemManager1{,/**}
     interface=org.freedesktop.DBus.*
     peer=(label=###PLUG_SECURITY_TAGS###),
-`)
+`
 
-var modemManagerConnectedPlugAppArmor = []byte(`
-# Description: Allow using ModemManager service. Reserved because this gives
-#  privileged access to the ModemManager service.
-# Usage: reserved
+const modemManagerConnectedPlugAppArmor = `
+# Description: Allow using ModemManager service. This gives privileged access
+# to the ModemManager service.
 
 #include <abstractions/dbus-strict>
 
@@ -114,9 +116,9 @@ dbus (receive, send)
     path=/org/freedesktop/ModemManager1{,/**}
     interface=org.freedesktop.DBus.*
     peer=(label=###SLOT_SECURITY_TAGS###),
-`)
+`
 
-var modemManagerConnectedPlugAppArmorClassic = []byte(`
+const modemManagerConnectedPlugAppArmorClassic = `
 # Allow access to the unconfined ModemManager service on classic.
 dbus (receive, send)
     bus=system
@@ -128,67 +130,35 @@ dbus (receive, send)
     path=/org/freedesktop/ModemManager1{,/**}
     interface=org.freedesktop.DBus.*
     peer=(label=unconfined),
-`)
+`
+
+const modemManagerPermanentSlotSecComp = `
+# Description: Allow operating as the ModemManager service. This gives
+# privileged access to the system.
 
-var modemManagerPermanentSlotSecComp = []byte(`
-# Description: Allow operating as the ModemManager service. Reserved because this
-#  gives privileged access to the system.
-# Usage: reserved
 # TODO: add ioctl argument filters when seccomp arg filtering is implemented
 accept
 accept4
 bind
-connect
-getpeername
-getsockname
-getsockopt
 listen
-recv
-recvfrom
-recvmmsg
-recvmsg
-send
-sendmmsg
-sendmsg
-sendto
-setsockopt
 shutdown
-socketpair
-socket
-`)
+`
 
-var modemManagerConnectedPlugSecComp = []byte(`
-# Description: Allow using ModemManager service. Reserved because this gives
-#  privileged access to the ModemManager service.
-# Usage: reserved
-
-# Can communicate with DBus system service
-connect
-getsockname
-recv
-recvmsg
-recvfrom
-send
-sendto
-sendmsg
-socket
-`)
-
-var modemManagerPermanentSlotDBus = []byte(`
+const modemManagerPermanentSlotDBus = `
 <policy user="root">
     <allow own="org.freedesktop.ModemManager1"/>
     <allow send_destination="org.freedesktop.ModemManager1"/>
 </policy>
-`)
+`
 
-var modemManagerConnectedPlugDBus = []byte(`
+const modemManagerConnectedPlugDBus = `
 <policy context="default">
     <deny own="org.freedesktop.ModemManager1"/>
     <deny send_destination="org.freedesktop.ModemManager1"/>
 </policy>
-`)
+`
 
-var modemManagerPermanentSlotUdev = []byte(`
+const modemManagerPermanentSlotUdev = `
 # Concatenation of all ModemManager udev rules
 # do not edit this file, it will be overwritten on update
 
@@ -1177,7 +1147,7 @@ KERNEL=="cdc-wdm*", SUBSYSTEM=="usb", ENV{ID_MM_CANDIDATE}="1"
 KERNEL=="cdc-wdm*", SUBSYSTEM=="usbmisc", ENV{ID_MM_CANDIDATE}="1"
 
 LABEL="mm_candidate_end"
-`)
+`
 
 type ModemManagerInterface struct{}
 
@@ -1202,8 +1172,6 @@ func (iface *ModemManagerInterface) ConnectedPlugSnippet(plug *interfaces.Plug,
 			snippet = append(snippet, modemManagerConnectedPlugAppArmorClassic...)
 		}
 		return snippet, nil
-	case interfaces.SecuritySecComp:
-		return modemManagerConnectedPlugSecComp, nil
 	}
 	return nil, nil
 }
@@ -1211,13 +1179,13 @@ func (iface *ModemManagerInterface) ConnectedPlugSnippet(plug *interfaces.Plug,
 func (iface *ModemManagerInterface) PermanentSlotSnippet(slot *interfaces.Slot, securitySystem interfaces.SecuritySystem) ([]byte, error) {
 	switch securitySystem {
 	case interfaces.SecurityAppArmor:
-		return modemManagerPermanentSlotAppArmor, nil
+		return []byte(modemManagerPermanentSlotAppArmor), nil
 	case interfaces.SecuritySecComp:
-		return modemManagerPermanentSlotSecComp, nil
+		return []byte(modemManagerPermanentSlotSecComp), nil
 	case interfaces.SecurityUDev:
-		return modemManagerPermanentSlotUdev, nil
+		return []byte(modemManagerPermanentSlotUdev), nil
 	case interfaces.SecurityDBus:
-		return modemManagerPermanentSlotDBus, nil
+		return []byte(modemManagerPermanentSlotDBus), nil
 	}
 	return nil, nil
 }
@@ -1227,7 +1195,7 @@ func (iface *ModemManagerInterface) ConnectedSlotSnippet(plug *interfaces.Plug,
 	case interfaces.SecurityAppArmor:
 		old := []byte("###PLUG_SECURITY_TAGS###")
 		new := plugAppLabelExpr(plug)
-		snippet := bytes.Replace(modemManagerConnectedSlotAppArmor, old, new, -1)
+		snippet := bytes.Replace([]byte(modemManagerConnectedSlotAppArmor), old, new, -1)
 		return snippet, nil
 	}
 	return nil, nil
