@@ -25,7 +25,6 @@ import (
 
 const openglConnectedPlugAppArmor = `
 # Description: Can access opengl.
-# Usage: reserved
 
   # specific gl libs
   /var/lib/snapd/lib/gl/ r,
@@ -38,6 +37,7 @@ const openglConnectedPlugAppArmor = `
   /dev/nvidiactl rw,
   /dev/nvidia-modeset rw,
   /dev/nvidia* rw,
+  unix (send, receive) type=dgram peer=(addr="@nvidia[0-9a-f]*"),
 
   # eglfs
   /dev/vchiq rw,
@@ -56,19 +56,11 @@ const openglConnectedPlugAppArmor = `
   /run/udev/data/c226:[0-9]* r,  # 226 drm
 `
 
-const openglConnectedPlugSecComp = `
-# Description: Can access opengl.
-# Usage: reserved
-
-getsockopt
-`
-
 // NewOpenglInterface returns a new "opengl" interface.
 func NewOpenglInterface() interfaces.Interface {
 	return &commonInterface{
 		name: "opengl",
 		connectedPlugAppArmor: openglConnectedPlugAppArmor,
-		connectedPlugSecComp:  openglConnectedPlugSecComp,
 		reservedForOS:         true,
 	}
 }
