@@ -22,6 +22,7 @@ package interfaces
 import (
 	"fmt"
 	"regexp"
+	"strings"
 
 	"github.com/snapcore/snapd/snap"
 )
@@ -43,6 +44,11 @@ type PlugRef struct {
 	Name string `json:"plug"`
 }
 
+// String returns the "snap:plug" representation of a plug reference.
+func (ref *PlugRef) String() string {
+	return fmt.Sprintf("%s:%s", ref.Snap, ref.Name)
+}
+
 // Slot represents a capacity offered by a snap.
 type Slot struct {
 	*snap.SlotInfo
@@ -60,6 +66,11 @@ type SlotRef struct {
 	Name string `json:"slot"`
 }
 
+// String returns the "snap:slot" representation of a slot reference.
+func (ref *SlotRef) String() string {
+	return fmt.Sprintf("%s:%s", ref.Snap, ref.Name)
+}
+
 // Interfaces holds information about a list of plugs and slots, and their connections.
 type Interfaces struct {
 	Plugs []*Plug `json:"plugs"`
@@ -77,6 +88,25 @@ func (conn *ConnRef) ID() string {
 	return fmt.Sprintf("%s:%s %s:%s", conn.PlugRef.Snap, conn.PlugRef.Name, conn.SlotRef.Snap, conn.SlotRef.Name)
 }
 
+// ParseConnRef parses an ID string
+func ParseConnRef(id string) (ConnRef, error) {
+	var conn ConnRef
+	parts := strings.SplitN(id, " ", 2)
+	if len(parts) != 2 {
+		return conn, fmt.Errorf("malformed connection identifier: %q", id)
+	}
+	plugParts := strings.Split(parts[0], ":")
+	slotParts := strings.Split(parts[1], ":")
+	if len(plugParts) != 2 || len(slotParts) != 2 {
+		return conn, fmt.Errorf("malformed connection identifier: %q", id)
+	}
+	conn.PlugRef.Snap = plugParts[0]
+	conn.PlugRef.Name = plugParts[1]
+	conn.SlotRef.Snap = slotParts[0]
+	conn.SlotRef.Name = slotParts[1]
+	return conn, nil
+}
+
 // Interface describes a group of interchangeable capabilities with common features.
 // Interfaces act as a contract between system builders, application developers
 // and end users.
@@ -157,6 +187,18 @@ type Interface interface {
 	AutoConnect(plug *Plug, slot *Slot) bool
 }
 
+// Specification describes interactions between backends and interfaces.
+type Specification interface {
+	// AddPermanentSlot records side-effects of having a slot.
+	AddPermanentSlot(iface Interface, slot *Slot) error
+	// AddPermanentPlug records side-effects of having a plug.
+	AddPermanentPlug(iface Interface, plug *Plug) error
+	// AddConnectedSlot records side-effects of having a connected slot.
+	AddConnectedSlot(iface Interface, plug *Plug, slot *Slot) error
+	// AddConnectedPlug records side-effects of having a connected plug.
+	AddConnectedPlug(iface Interface, plug *Plug, slot *Slot) error
+}
+
 // SecuritySystem is a name of a security system.
 type SecuritySystem string
 
