@@ -42,7 +42,7 @@ import (
 type Backend struct{}
 
 // Name returns the name of the backend.
-func (b *Backend) Name() string {
+func (b *Backend) Name() interfaces.SecuritySystem {
 	return "dbus"
 }
 
@@ -131,3 +131,7 @@ func addContent(securityTag string, executableSnippets [][]byte, content map[str
 		Mode:    0644,
 	}
 }
+
+func (b *Backend) NewSpecification() interfaces.Specification {
+	panic(fmt.Errorf("%s is not using specifications yet", b.Name()))
+}
