@@ -1,7 +1,7 @@
 // -*- Mode: Go; indent-tabs-mode: t -*-
 
 /*
- * Copyright (C) 2016 Canonical Ltd
+ * Copyright (C) 2016-2017 Canonical Ltd
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -20,53 +20,25 @@
 package seccomp
 
 // defaultTemplate contains default seccomp template.
-//
 // It can be overridden for testing using MockTemplate().
-//
-// http://bazaar.launchpad.net/~ubuntu-security/ubuntu-core-security/trunk/view/head:/data/seccomp/templates/ubuntu-core/16.04/default
 var defaultTemplate = []byte(`
 # Description: Allows access to app-specific directories and basic runtime
-# Usage: common
 #
+# The default seccomp policy is default deny with a whitelist of allowed
+# syscalls. The default policy is intended to be safe for any application to
+# use and should be evaluated in conjunction with other security backends (eg
+# AppArmor). For example, a few particularly problematic syscalls that are left
+# out of the default policy are (non-exhaustive):
+# - kexec_load
+# - create_module, init_module, finit_module, delete_module (kernel modules)
+# - name_to_handle_at (history of vulnerabilities)
+# - open_by_handle_at (history of vulnerabilities)
+# - ptrace (can be used to break out of sandbox with <4.8 kernels)
+# - add_key, keyctl, request_key (kernel keyring)
 
-# Dangerous syscalls that we don't ever want to allow.
-# Note: may uncomment once ubuntu-core-launcher understands @deny rules and
-# if/when we conditionally deny these in the future.
-
-# kexec
-#@deny kexec_load
-
-# kernel modules
-#@deny create_module
-#@deny init_module
-#@deny finit_module
-#@deny delete_module
-
-# these have a history of vulnerabilities, are not widely used, and
-# open_by_handle_at has been used to break out of docker containers by brute
-# forcing the handle value: http://stealth.openwall.net/xSports/shocker.c
-#@deny name_to_handle_at
-#@deny open_by_handle_at
-
-# Explicitly deny ptrace since it can be abused to break out of the seccomp
-# sandbox
-#@deny ptrace
-
-# Explicitly deny capability mknod so apps can't create devices
-#@deny mknod
-#@deny mknodat
-
-# Explicitly deny (u)mount so apps can't change mounts in their namespace
-#@deny mount
-#@deny umount
-#@deny umount2
-
-# Explicitly deny kernel keyring access
-#@deny add_key
-#@deny keyctl
-#@deny request_key
-
-# end dangerous syscalls
+#
+# Allowed accesses
+#
 
 access
 faccessat
@@ -95,22 +67,24 @@ chmod
 fchmod
 fchmodat
 
-# snappy doesn't currently support per-app UID/GIDs so don't allow chown. To
-# properly support chown, we need to have syscall arg filtering (LP: #1446748)
-# and per-app UID/GIDs.
-#chown
-#chown32
-#fchown
-#fchown32
-#fchownat
-#lchown
-#lchown32
+# snappy doesn't currently support per-app UID/GIDs. All daemons run as 'root'
+# so allow chown to 'root'. DAC will prevent non-root from chowning to root.
+chown - 0 0
+chown32 - 0 0
+fchown - 0 0
+fchown32 - 0 0
+lchown - 0 0
+lchown32 - 0 0
 
 clock_getres
 clock_gettime
 clock_nanosleep
 clone
 close
+
+# needed by ls -l
+connect
+
 creat
 dup
 dup2
@@ -187,7 +161,11 @@ inotify_init
 inotify_init1
 inotify_rm_watch
 
-# Needed by shell
+# TIOCSTI allows for faking input (man tty_ioctl)
+# TODO: this should be scaled back even more
+#ioctl - !TIOCSTI
+# FIXME: replace this with the filter of TIOCSTI once snap-confine can read this syntax
+# See LP:#1662489 for context.
 ioctl
 
 io_cancel
@@ -254,10 +232,16 @@ munmap
 
 nanosleep
 
-# LP: #1446748 - deny until we have syscall arg filtering. Alternatively, set
-# RLIMIT_NICE hard limit for apps, launch them under an appropriate nice value
-# and allow this call
-#nice
+# Allow using nice() with default or lower priority
+# FIXME: https://github.com/seccomp/libseccomp/issues/69 which means we
+# currently have to use <=19. When that bug is fixed, use >=0
+nice <=19
+# Allow using setpriority to set the priority of the calling process to default
+# or lower priority (eg, 'nice -n 9 <command>')
+# default or lower priority.
+# FIXME: https://github.com/seccomp/libseccomp/issues/69 which means we
+# currently have to use <=19. When that bug is fixed, use >=0
+setpriority PRIO_PROCESS 0 <=19
 
 # LP: #1446748 - support syscall arg filtering for mode_t with O_CREAT
 open
@@ -284,6 +268,13 @@ readahead
 readdir
 readlink
 readlinkat
+
+# allow reading from sockets
+recv
+recvfrom
+recvmsg
+recvmmsg
+
 remap_file_pages
 
 removexattr
@@ -314,8 +305,14 @@ sched_get_priority_max
 sched_get_priority_min
 sched_getscheduler
 sched_rr_get_interval
-# LP: #1446748 - when support syscall arg filtering, enforce pid_t is 0 so the
-# app may only change its own scheduler
+# enforce pid_t is 0 so the app may only change its own scheduler and affinity.
+# Use process-control interface for controlling other pids.
+sched_setaffinity 0 - -
+sched_setparam 0 -
+
+# 'sched_setscheduler' without argument filtering was allowed in 2.21 and
+# earlier and 2.22 added 'sched_setscheduler 0 - -', introducing LP: #1661265.
+# For now, continue to allow sched_setscheduler unconditionally.
 sched_setscheduler
 
 sched_yield
@@ -334,6 +331,13 @@ semctl
 semget
 semop
 semtimedop
+
+# allow sending to sockets
+send
+sendto
+sendmsg
+sendmmsg
+
 sendfile
 sendfile64
 
@@ -394,11 +398,13 @@ sigsuspend
 sigtimedwait
 sigwaitinfo
 
-# needed by ls -l
+# AppArmor mediates AF_UNIX/AF_LOCAL via 'unix' rules and all other AF_*
+# domains via 'network' rules. We won't allow bare 'network' AppArmor rules, so
+# we can allow 'socket' for any domain and let AppArmor handle the rest.
 socket
-connect
 
 # needed by snapctl
+getsockopt
 setsockopt
 getsockname
 getpeername
