@@ -20,8 +20,10 @@
 package osutil
 
 import (
+	"fmt"
 	"os"
 	"strconv"
+	"strings"
 )
 
 // GetenvBool returns whether the given key may be considered "set" in the
@@ -50,3 +52,39 @@ func GetenvBool(key string, dflt ...bool) bool {
 
 	return b
 }
+
+// SubstituteEnv takes a list of environment strings like:
+// - K1=BAR
+// - K2=$K1
+// - K3=${K2}
+// and substitutes them top-down from the given environment
+// and from the os environment.
+//
+// Input strings that do not have the form "k=v" will be dropped
+// from the output.
+//
+// The result will be a list of environment strings in the same
+// order as the input.
+func SubstituteEnv(env []string) []string {
+	envMap := map[string]string{}
+	out := make([]string, 0, len(env))
+
+	for _, s := range env {
+		l := strings.SplitN(s, "=", 2)
+		if len(l) < 2 {
+			continue
+		}
+		k := l[0]
+		v := l[1]
+		v = os.Expand(v, func(k string) string {
+			if s, ok := envMap[k]; ok {
+				return s
+			}
+			return os.ExpandEnv(v)
+		})
+		out = append(out, fmt.Sprintf("%s=%s", k, v))
+		envMap[k] = v
+	}
+
+	return out
+}
