@@ -30,8 +30,6 @@ import (
 	"fmt"
 	"net/http"
 	"net/url"
-	"os"
-	"path/filepath"
 	"regexp"
 	"strings"
 	"time"
@@ -40,11 +38,13 @@ import (
 
 	"github.com/snapcore/snapd/asserts"
 	"github.com/snapcore/snapd/dirs"
+	"github.com/snapcore/snapd/httputil"
 	"github.com/snapcore/snapd/i18n/dumb"
+	"github.com/snapcore/snapd/logger"
 	"github.com/snapcore/snapd/osutil"
 	"github.com/snapcore/snapd/overlord/assertstate"
 	"github.com/snapcore/snapd/overlord/auth"
-	"github.com/snapcore/snapd/overlord/configstate"
+	"github.com/snapcore/snapd/overlord/configstate/config"
 	"github.com/snapcore/snapd/overlord/hookstate"
 	"github.com/snapcore/snapd/overlord/snapstate"
 	"github.com/snapcore/snapd/overlord/state"
@@ -59,7 +59,12 @@ type DeviceManager struct {
 	state      *state.State
 	keypairMgr asserts.KeypairManager
 	runner     *state.TaskRunner
-	bootOkRan  bool
+
+	bootOkRan            bool
+	bootRevisionsUpdated bool
+
+	lastBecomeOperationalAttempt time.Time
+	becomeOperationalBackoff     time.Duration
 }
 
 // Manager returns a new device manager.
@@ -111,6 +116,41 @@ func (m *DeviceManager) changeInFlight(kind string) bool {
 	return false
 }
 
+// helpers to keep count of attempts to get a serial, useful to decide
+// to give up holding off trying to auto-refresh
+
+type ensureOperationalAttemptsKey struct{}
+
+func incEnsureOperationalAttempts(st *state.State) {
+	cur, _ := st.Cached(ensureOperationalAttemptsKey{}).(int)
+	st.Cache(ensureOperationalAttemptsKey{}, cur+1)
+}
+
+func ensureOperationalAttempts(st *state.State) int {
+	cur, _ := st.Cached(ensureOperationalAttemptsKey{}).(int)
+	return cur
+}
+
+// ensureOperationalShouldBackoff returns whether we should abstain from
+// further become-operational tentatives while its backoff interval is
+// not expired.
+func (m *DeviceManager) ensureOperationalShouldBackoff(now time.Time) bool {
+	if !m.lastBecomeOperationalAttempt.IsZero() && m.lastBecomeOperationalAttempt.Add(m.becomeOperationalBackoff).After(now) {
+		return true
+	}
+	if m.becomeOperationalBackoff == 0 {
+		m.becomeOperationalBackoff = 5 * time.Minute
+	} else {
+		newBackoff := m.becomeOperationalBackoff * 2
+		if newBackoff > (12 * time.Hour) {
+			newBackoff = 24 * time.Hour
+		}
+		m.becomeOperationalBackoff = newBackoff
+	}
+	m.lastBecomeOperationalAttempt = now
+	return false
+}
+
 func (m *DeviceManager) ensureOperational() error {
 	m.state.Lock()
 	defer m.state.Unlock()
@@ -126,13 +166,12 @@ func (m *DeviceManager) ensureOperational() error {
 	}
 
 	if device.Brand == "" || device.Model == "" {
-		// need first-boot, loading of model assertion info
-		if release.OnClassic {
-			// TODO: are we going to have model assertions on classic or need will need to cheat here?
-			return nil
-		}
-		// cannot proceed yet, once first boot is done these will be set
-		// and we can pick up from there
+		// cannot proceed until seeding has loaded the model
+		// assertion and set the brand and model, that is
+		// optional on classic
+		// TODO: later we can check if
+		// "seeded" was set and we still don't have a brand/model
+		// and use a fallback assertion
 		return nil
 	}
 
@@ -140,11 +179,9 @@ func (m *DeviceManager) ensureOperational() error {
 		return nil
 	}
 
-	if serialRequestURL == "" {
-		// cannot do anything actually
-		return nil
-	}
-
+	// TODO: make presence of gadget optional on classic? that is
+	// sensible only for devices that the store can give directly
+	// serials to and when we will have a general fallback
 	gadgetInfo, err := snapstate.GadgetInfo(m.state)
 	if err == state.ErrNoState {
 		// no gadget installed yet, cannot proceed
@@ -154,6 +191,13 @@ func (m *DeviceManager) ensureOperational() error {
 		return err
 	}
 
+	// have some backoff between full retries
+	if m.ensureOperationalShouldBackoff(time.Now()) {
+		return nil
+	}
+	// increment attempt count
+	incEnsureOperationalAttempts(m.state)
+
 	// XXX: some of these will need to be split and use hooks
 	// retries might need to embrace more than one "task" then,
 	// need to be careful
@@ -194,17 +238,6 @@ func (m *DeviceManager) ensureSeedYaml() error {
 	m.state.Lock()
 	defer m.state.Unlock()
 
-	// FIXME: enable on classic?
-	//
-	// Disable seed.yaml on classic for now. In the long run we want
-	// classic to have a seed parsing as well so that we can install
-	// snaps in a classic environment (LP: #1609903). However right
-	// now it is under heavy development so until the dust
-	// settles we disable it.
-	if release.OnClassic {
-		return nil
-	}
-
 	var seeded bool
 	err := m.state.Get("seeded", &seeded)
 	if err != nil && err != state.ErrNoState {
@@ -218,12 +251,6 @@ func (m *DeviceManager) ensureSeedYaml() error {
 		return nil
 	}
 
-	coreInfo, err := snapstate.CoreInfo(m.state)
-	if err == nil && coreInfo.Name() == "ubuntu-core" {
-		// already seeded... recover
-		return m.alreadyFirstbooted()
-	}
-
 	tsAll, err := populateStateFromSeed(m.state)
 	if err != nil {
 		return err
@@ -242,52 +269,6 @@ func (m *DeviceManager) ensureSeedYaml() error {
 	return nil
 }
 
-// alreadyFirstbooted recovers already first booted devices with the old method appropriately
-func (m *DeviceManager) alreadyFirstbooted() error {
-	device, err := auth.Device(m.state)
-	if err != nil {
-		return err
-	}
-	// recover key-id
-	if device.Brand != "" && device.Model != "" {
-		serials, err := assertstate.DB(m.state).FindMany(asserts.SerialType, map[string]string{
-			"brand-id": device.Brand,
-			"model":    device.Model,
-		})
-		if err != nil && err != asserts.ErrNotFound {
-			return err
-		}
-
-		if len(serials) == 1 {
-			// we can recover the key id from the assertion
-			serial := serials[0].(*asserts.Serial)
-			keyID := serial.DeviceKey().ID()
-			device.KeyID = keyID
-			device.Serial = serial.Serial()
-			err := auth.SetDevice(m.state, device)
-			if err != nil {
-				return err
-			}
-			// best effort to cleanup abandoned keys
-			pat := filepath.Join(dirs.SnapDeviceDir, "private-keys-v1", "*")
-			keyFns, err := filepath.Glob(pat)
-			if err != nil {
-				panic(fmt.Sprintf("invalid glob for device keys: %v", err))
-			}
-			for _, keyFn := range keyFns {
-				if filepath.Base(keyFn) == keyID {
-					continue
-				}
-				os.Remove(keyFn)
-			}
-		}
-
-	}
-
-	m.state.Set("seeded", true)
-	return nil
-}
-
 func (m *DeviceManager) ensureBootOk() error {
 	m.state.Lock()
 	defer m.state.Unlock()
@@ -307,7 +288,14 @@ func (m *DeviceManager) ensureBootOk() error {
 		m.bootOkRan = true
 	}
 
-	return snapstate.UpdateBootRevisions(m.state)
+	if !m.bootRevisionsUpdated {
+		if err := snapstate.UpdateBootRevisions(m.state); err != nil {
+			return err
+		}
+		m.bootRevisionsUpdated = true
+	}
+
+	return nil
 }
 
 type ensureError struct {
@@ -446,6 +434,33 @@ func retryErr(t *state.Task, reason string, a ...interface{}) error {
 	return &state.Retry{After: retryInterval}
 }
 
+type serverError struct {
+	Message string         `json:"message"`
+	Errors  []*serverError `json:"error_list"`
+}
+
+func retryBadStatus(t *state.Task, reason string, resp *http.Response) error {
+	if resp.StatusCode > http.StatusInternalServerError {
+		// likely temporary
+		return retryErr(t, "%s: unexpected status %d", reason, resp.StatusCode)
+	}
+	if resp.Header.Get("Content-Type") == "application/json" {
+		var srvErr serverError
+		dec := json.NewDecoder(resp.Body)
+		err := dec.Decode(&srvErr)
+		if err == nil {
+			msg := srvErr.Message
+			if msg == "" && len(srvErr.Errors) > 0 {
+				msg = srvErr.Errors[0].Message
+			}
+			if msg != "" {
+				return fmt.Errorf("%s: %s", reason, msg)
+			}
+		}
+	}
+	return fmt.Errorf("%s: unexpected status %d", reason, resp.StatusCode)
+}
+
 func prepareSerialRequest(t *state.Task, privKey asserts.PrivateKey, device *auth.DeviceState, client *http.Client, cfg *serialRequestConfig) (string, error) {
 	st := t.State()
 	st.Unlock()
@@ -455,6 +470,7 @@ func prepareSerialRequest(t *state.Task, privKey asserts.PrivateKey, device *aut
 	if err != nil {
 		return "", fmt.Errorf("internal error: cannot create request-id request %q", cfg.requestIDURL)
 	}
+	req.Header.Set("User-Agent", httputil.UserAgent())
 	cfg.applyHeaders(req)
 
 	resp, err := client.Do(req)
@@ -463,7 +479,7 @@ func prepareSerialRequest(t *state.Task, privKey asserts.PrivateKey, device *aut
 	}
 	defer resp.Body.Close()
 	if resp.StatusCode != 200 {
-		return "", retryErr(t, "cannot retrieve request-id for making a request for a serial: unexpected status %d", resp.StatusCode)
+		return "", retryBadStatus(t, "cannot retrieve request-id for making a request for a serial", resp)
 	}
 
 	dec := json.NewDecoder(resp.Body)
@@ -508,6 +524,7 @@ func submitSerialRequest(t *state.Task, serialRequest string, client *http.Clien
 	if err != nil {
 		return nil, fmt.Errorf("internal error: cannot create serial-request request %q", cfg.serialRequestURL)
 	}
+	req.Header.Set("User-Agent", httputil.UserAgent())
 	cfg.applyHeaders(req)
 	req.Header.Set("Content-Type", asserts.MediaType)
 
@@ -522,7 +539,7 @@ func submitSerialRequest(t *state.Task, serialRequest string, client *http.Clien
 	case 202:
 		return nil, errPoll
 	default:
-		return nil, retryErr(t, "cannot deliver device serial request: unexpected status %d", resp.StatusCode)
+		return nil, retryBadStatus(t, "cannot deliver device serial request", resp)
 	}
 
 	// decode body with serial assertion
@@ -556,7 +573,10 @@ func getSerial(t *state.Task, privKey asserts.PrivateKey, device *auth.DeviceSta
 		return a.(*asserts.Serial), nil
 	}
 
-	client := &http.Client{Timeout: 30 * time.Second}
+	client := httputil.NewHTTPClient(&httputil.ClientOpts{
+		Timeout:    30 * time.Second,
+		MayLogBody: true,
+	})
 
 	// NB: until we get at least an Accepted (202) we need to
 	// retry from scratch creating a new request-id because the
@@ -618,7 +638,7 @@ func getSerialRequestConfig(t *state.Task) (*serialRequestConfig, error) {
 	}
 	gadgetName := gadgetInfo.Name()
 
-	tr := configstate.NewTransaction(t.State())
+	tr := config.NewTransaction(t.State())
 	var svcURL string
 	err = tr.GetMaybe(gadgetName, "device-service.url", &svcURL)
 	if err != nil {
@@ -782,6 +802,44 @@ func (m *DeviceManager) doMarkSeeded(t *state.Task, _ *tomb.Tomb) error {
 	return nil
 }
 
+// canAutoRefresh is a helper that checks if the device is able to
+// auto-refresh
+func canAutoRefresh(st *state.State) (bool, error) {
+	// we need to be seeded first
+	var seeded bool
+	st.Get("seeded", &seeded)
+	if !seeded {
+		return false, nil
+	}
+
+	_, err := Model(st)
+	if err == state.ErrNoState {
+		// no model, no need to wait for a serial
+		// can happen only on classic
+		return true, nil
+	}
+	if err != nil {
+		return false, err
+	}
+
+	// either we have a serial or we try anyway if we attempted
+	// for a while to get a serial, this would allow us to at
+	// least upgrade core if that can help
+	if ensureOperationalAttempts(st) >= 3 {
+		return true, nil
+	}
+
+	_, err = Serial(st)
+	if err == state.ErrNoState {
+		return false, nil
+	}
+	if err != nil {
+		return false, err
+	}
+
+	return true, nil
+}
+
 var repeatRequestSerial string
 
 // implementing auth.DeviceAssertions
@@ -896,6 +954,10 @@ func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, flags sn
 		currentInfo = snapstate.GadgetInfo
 		getName = (*asserts.Model).Gadget
 	case snap.TypeKernel:
+		if release.OnClassic {
+			return fmt.Errorf("cannot install a kernel snap on classic")
+		}
+
 		kind = "kernel"
 		currentInfo = snapstate.KernelInfo
 		getName = (*asserts.Model).Kernel
@@ -904,9 +966,25 @@ func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, flags sn
 		return nil
 	}
 
-	if release.OnClassic {
-		// for the time being
-		return fmt.Errorf("cannot install a %s snap on classic", kind)
+	model, err := Model(st)
+	if err == state.ErrNoState {
+		return fmt.Errorf("cannot install %s without model assertion", kind)
+	}
+	if err != nil {
+		return err
+	}
+
+	if snapInfo.SnapID != "" {
+		snapDecl, err := assertstate.SnapDeclaration(st, snapInfo.SnapID)
+		if err != nil {
+			return fmt.Errorf("internal error: cannot find snap declaration for %q: %v", snapInfo.Name(), err)
+		}
+		publisher := snapDecl.PublisherID()
+		if publisher != "canonical" && publisher != model.BrandID() {
+			return fmt.Errorf("cannot install %s %q published by %q for model by %q", kind, snapInfo.Name(), publisher, model.BrandID())
+		}
+	} else {
+		logger.Noticef("installing unasserted %s %q", kind, snapInfo.Name())
 	}
 
 	currentSnap, err := currentInfo(st)
@@ -919,15 +997,11 @@ func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, flags sn
 	}
 	// first installation of a gadget/kernel
 
-	model, err := Model(st)
-	if err == state.ErrNoState {
-		return fmt.Errorf("cannot install %s without model assertion", kind)
-	}
-	if err != nil {
-		return err
+	expectedName := getName(model)
+	if expectedName == "" { // can happen only on classic
+		return fmt.Errorf("cannot install %s snap on classic if not requested by the model", kind)
 	}
 
-	expectedName := getName(model)
 	if snapInfo.Name() != expectedName {
 		return fmt.Errorf("cannot install %s %q, model assertion requests %q", kind, snapInfo.Name(), expectedName)
 	}
@@ -937,4 +1011,5 @@ func checkGadgetOrKernel(st *state.State, snapInfo, curInfo *snap.Info, flags sn
 
 func init() {
 	snapstate.AddCheckSnapCallback(checkGadgetOrKernel)
+	snapstate.CanAutoRefresh = canAutoRefresh
 }
