@@ -1,5 +1,3 @@
-// -*- Mode: Go; indent-tabs-mode: t -*-
-
 /*
  * Copyright (C) 2016 Canonical Ltd
  *
@@ -18,3 +16,21 @@
  */
 
 package ifacestate
+
+import (
+	"errors"
+
+	"gopkg.in/tomb.v2"
+
+	"github.com/snapcore/snapd/overlord/state"
+)
+
+// AddForeignTaskHandlers registers handlers for tasks handled outside of the
+// InterfaceManager.
+func (m *InterfaceManager) AddForeignTaskHandlers() {
+	// Add handler to test full aborting of changes
+	erroringHandler := func(task *state.Task, _ *tomb.Tomb) error {
+		return errors.New("error out")
+	}
+	m.runner.AddHandler("error-trigger", erroringHandler, nil)
+}
