@@ -22,7 +22,6 @@ package snapstate_test
 import (
 	"errors"
 	"fmt"
-	"strings"
 
 	"golang.org/x/net/context"
 
@@ -39,11 +38,12 @@ import (
 type fakeOp struct {
 	op string
 
-	name  string
-	revno snap.Revision
-	sinfo snap.SideInfo
-	stype snap.Type
-	cand  store.RefreshCandidate
+	name    string
+	channel string
+	revno   snap.Revision
+	sinfo   snap.SideInfo
+	stype   snap.Type
+	cand    store.RefreshCandidate
 
 	old string
 
@@ -120,9 +120,15 @@ func (f *fakeStore) SnapInfo(spec store.SnapSpec, user *auth.UserState) (*snap.I
 		confinement = snap.ClassicConfinement
 	}
 
+	typ := snap.TypeApp
+	if spec.Name == "some-core" {
+		typ = snap.TypeOS
+	}
+
 	info := &snap.Info{
+		Architectures: []string{"all"},
 		SideInfo: snap.SideInfo{
-			RealName: strings.Split(spec.Name, ".")[0],
+			RealName: spec.Name,
 			Channel:  spec.Channel,
 			SnapID:   "snapIDsnapidsnapidsnapidsnapidsn",
 			Revision: spec.Revision,
@@ -132,6 +138,7 @@ func (f *fakeStore) SnapInfo(spec store.SnapSpec, user *auth.UserState) (*snap.I
 			DownloadURL: "https://some-server.com/some/path.snap",
 		},
 		Confinement: confinement,
+		Type:        typ,
 	}
 	f.fakeBackend.ops = append(f.fakeBackend.ops, fakeOp{op: "storesvc-snap", name: spec.Name, revno: spec.Revision})
 
@@ -165,9 +172,12 @@ func (f *fakeStore) ListRefresh(cands []*store.RefreshCandidate, _ *auth.UserSta
 		}
 
 		var name string
-		if snapID == "some-snap-id" {
+		switch snapID {
+		case "some-snap-id":
 			name = "some-snap"
-		} else {
+		case "core-snap-id":
+			name = "core"
+		default:
 			panic(fmt.Sprintf("ListRefresh: unknown snap-id: %s", snapID))
 		}
 
@@ -193,7 +203,8 @@ func (f *fakeStore) ListRefresh(cands []*store.RefreshCandidate, _ *auth.UserSta
 			DownloadInfo: snap.DownloadInfo{
 				DownloadURL: "https://some-server.com/some/path.snap",
 			},
-			Confinement: confinement,
+			Confinement:   confinement,
+			Architectures: []string{"all"},
 		}
 
 		var hit snap.Revision
@@ -298,7 +309,11 @@ func (f *fakeSnappyBackend) ReadInfo(name string, si *snap.SideInfo) (*snap.Info
 		return nil, errors.New(`cannot read info for "borken" snap`)
 	}
 	// naive emulation for now, always works
-	info := &snap.Info{SuggestedName: name, SideInfo: *si}
+	info := &snap.Info{
+		SuggestedName: name,
+		SideInfo:      *si,
+		Architectures: []string{"all"},
+	}
 	info.Type = snap.TypeApp
 	if name == "gadget" {
 		info.Type = snap.TypeGadget
