@@ -209,6 +209,15 @@ func (bs *bootedSuite) TestUpdateBootRevisionsOSErrorsLate(c *C) {
 	st.Lock()
 	defer st.Unlock()
 
+	// have a kernel
+	snaptest.MockSnap(c, "name: canonical-pc-linux\ntype: os\nversion: 2", "", kernelSI2)
+	snapstate.Set(st, "canonical-pc-linux", &snapstate.SnapState{
+		SnapType: "kernel",
+		Active:   true,
+		Sequence: []*snap.SideInfo{kernelSI2},
+		Current:  snap.R(2),
+	})
+
 	// put core into the state but add no files on disk
 	// will break in the tasks
 	snapstate.Set(st, "core", &snapstate.SnapState{
@@ -219,7 +228,7 @@ func (bs *bootedSuite) TestUpdateBootRevisionsOSErrorsLate(c *C) {
 	})
 	bs.fakeBackend.linkSnapFailTrigger = filepath.Join(dirs.SnapMountDir, "/core/1")
 
-	bs.bootloader.BootVars["snap_kernel"] = "core_1.snap"
+	bs.bootloader.BootVars["snap_core"] = "core_1.snap"
 	err := snapstate.UpdateBootRevisions(st)
 	c.Assert(err, IsNil)
 
@@ -245,3 +254,24 @@ func (bs *bootedSuite) TestNameAndRevnoFromSnapInvalidFormat(c *C) {
 	_, _, err := snapstate.NameAndRevnoFromSnap("invalid")
 	c.Assert(err, ErrorMatches, `input "invalid" has invalid format \(not enough '_'\)`)
 }
+
+func (bs *bootedSuite) TestCurrentBootNameAndRevision(c *C) {
+	name, revision, err := snapstate.CurrentBootNameAndRevision(snap.TypeOS)
+	c.Check(err, IsNil)
+	c.Check(name, Equals, "core")
+	c.Check(revision, Equals, snap.R(2))
+
+	name, revision, err = snapstate.CurrentBootNameAndRevision(snap.TypeKernel)
+	c.Check(err, IsNil)
+	c.Check(name, Equals, "canonical-pc-linux")
+	c.Check(revision, Equals, snap.R(2))
+
+	bs.bootloader.BootVars["snap_mode"] = "trying"
+	_, _, err = snapstate.CurrentBootNameAndRevision(snap.TypeKernel)
+	c.Check(err, Equals, snapstate.ErrBootNameAndRevisionAgain)
+
+	bs.bootloader.BootVars["snap_mode"] = ""
+	delete(bs.bootloader.BootVars, "snap_kernel")
+	_, _, err = snapstate.CurrentBootNameAndRevision(snap.TypeKernel)
+	c.Check(err, ErrorMatches, "cannot retrieve boot revision for kernel: unset")
+}
