@@ -70,14 +70,15 @@ func snapEnv(info *snap.Info) map[string]string {
 // somewhere more reasonable like the snappy module.
 func basicEnv(info *snap.Info) map[string]string {
 	return map[string]string{
-		"SNAP":              info.MountDir(),
-		"SNAP_COMMON":       info.CommonDataDir(),
-		"SNAP_DATA":         info.DataDir(),
-		"SNAP_NAME":         info.Name(),
-		"SNAP_VERSION":      info.Version,
-		"SNAP_REVISION":     info.Revision.String(),
-		"SNAP_ARCH":         arch.UbuntuArchitecture(),
-		"SNAP_LIBRARY_PATH": "/var/lib/snapd/lib/gl:",
+		"SNAP":          info.MountDir(),
+		"SNAP_COMMON":   info.CommonDataDir(),
+		"SNAP_DATA":     info.DataDir(),
+		"SNAP_NAME":     info.Name(),
+		"SNAP_VERSION":  info.Version,
+		"SNAP_REVISION": info.Revision.String(),
+		"SNAP_ARCH":     arch.UbuntuArchitecture(),
+		// see https://github.com/snapcore/snapd/pull/2732#pullrequestreview-18827193
+		"SNAP_LIBRARY_PATH": "/var/lib/snapd/lib/gl:/var/lib/snapd/void",
 		"SNAP_REEXEC":       os.Getenv("SNAP_REEXEC"),
 	}
 }
