@@ -44,10 +44,15 @@ type snapDetails struct {
 	Revision         int                `json:"revision"` // store revisions are ints starting at 1
 	ScreenshotURLs   []string           `json:"screenshot_urls,omitempty"`
 	SnapID           string             `json:"snap_id"`
-	SupportURL       string             `json:"support_url"`
-	Title            string             `json:"title"`
-	Type             snap.Type          `json:"content,omitempty"`
-	Version          string             `json:"version"`
+
+	// FIXME: the store should send "contact" here, once it does we
+	//        can remove support_url
+	SupportURL string `json:"support_url"`
+	Contact    string `json:"contact"`
+
+	Title   string    `json:"title"`
+	Type    snap.Type `json:"content,omitempty"`
+	Version string    `json:"version"`
 
 	// TODO: have the store return a 'developer_username' for this
 	Developer   string `json:"origin"`
